
! --------------------------- BEGIN GET_LUN ----------------
      subroutine get_lun(lun)

* Finds a logical unit number not in use; returns
* -1 if it cannot find one.

* Dates -- 05/19/98 - Written by D. Boore, following
*                     Larry Baker's suggestion

      logical isopen
      do i = 99,10,-1
        inquire (unit=i, opened=isopen)
        if(.not.isopen) then
          lun = i
          return
        end if
      end do
      lun = -1

      return
      end
! --------------------------- END GET_LUN ----------------
     
! ------------------------------------------------------------------ skipcmnt
      subroutine skipcmnt(nu, comment, ncomments)

* Skip text comments in the file attached to unit nu, but save skipped 
* comments in character array comment.  Skip at least one line, and more as 
* long as the lines are preceded by "|" or "!".

* Dates: 04/16/01 - Written by D. Boore
*        12/07/01 - Added check for eof
*        11/04/03 - Use trim_c to trim possible leading blank
*        02/03/07 - Initialize comments to blank

      character comment(*)*(*), buf*80

      ncomments = 0
100   buf = ' '
      read (nu,'(a)',end=999) buf
      call trim_c(buf,nc_buf)
      if (buf(1:1) .eq.'!' .or. buf(1:1) .eq.'|' .or. 
     :                     ncomments + 1 .eq. 1) then
        ncomments = ncomments + 1
        comment(ncomments) = ' '
        comment(ncomments) = buf(1:nc_buf)
        goto 100
      else 
        backspace nu
      end if

999   continue
 
      return
      end
! ------------------------------------------------------------------ skipcmnt
! ---------------------- BEGIN SKIP -------------------
      subroutine SKIP(lunit, nlines)
        if (nlines .lt. 1) then
          return
        else
          do i = 1, nlines
             read(lunit, *)
          end do
          return
        end if
      end
! ---------------------- END SKIP -------------------
! --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
* Converts character string in TEXT to uppercase
* Dates: 03/12/96 - Written by Larry Baker

C
      Implicit   None
C
      Character  text*(*)
C
      Integer    j
      Character  ch
C
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
C
      Return
      End
! --------------------- END UPSTR ----------------------------------
! --------------------------- BEGIN TRIM_C -----------------------
      subroutine trim_c(cstr, nchar)

* strips leading and trailing blanks from cstr, returning the
* result in cstr, which is now nchar characters in length

* Strip off tabs also.

* Here is a sample use in constructing a column header, filled out with 
* periods:

** Read idtag:
*        idtag = ' '
*        read(nu_in, '(1x,a)') idtag
*        call trim_c(idtag, nc_id)
** Set up the column headings:
*        colhead = ' '
*        colhead = idtag(1:nc_id)//'......' ! nc_id + 6 > length of colhead

* Dates: 12/23/97 - written by D. Boore
*        12/08/00 - pad with trailing blanks.  Otherwise some comparisons
*                   of the trimmed character string with other strings
*                   can be in error because the original characters are left
*                   behind after shifting.  For example, here is a string
*                   before and after shifting, using the old version:
*                      col:12345
*                           MTWH  before
*                          MTWHH  after (but nc = 4).
*        03/21/01 - Check for a zero length input string
*        11/09/01 - Change check for zero length string to check for all blanks
*        10/19/09 - Strip off tabs

      character cstr*(*)

      if(cstr .eq. ' ') then
        nchar = 0
        return
      end if

      nend = len(cstr)

! Replace tabs with blanks:

      do i = 1, nend
        if(ichar(cstr(i:i)) .eq. 9) then
           cstr(i:i) = ' '
        end if
      end do



*      if(nend .eq. 0) then
*        nchar = 0
*        return
*      end if

      do i = nend, 1, -1
        if (cstr(i:i) .ne. ' ') then
           nchar2 = i
           goto 10
        end if
      end do

10    continue

      do j = 1, nchar2
        if (cstr(j:j) .ne. ' ') then
          nchar1 = j
          goto 20
        end if
      end do

20    continue
   
      nchar = nchar2 - nchar1 + 1
      cstr(1:nchar) = cstr(nchar1: nchar2)
      if (nchar .lt. nend) then
        do i = nchar+1, nend
          cstr(i:i) = ' '
        end do
      end if

      return
      end
! --------------------------- END TRIM_C -----------------------

! begin subroutine interpolate
      subroutine interpolate(y1, y2, x1, x2, x, y)
      
* Dates: 01/12/10 - Written by D. Boore

      real y1, y2, x1, x2, x, y, slope

      slope = (y2 - y1)/(x2-x1)
      y  = y1 + slope*(x - x1)
      
      return
      end
! end subroutine interpolate
      
      subroutine lin_interp(x, y, n, j, x_intrp, y_intrp)
      
* Computes linearly interpolated value of y

* Values out of range are assigned end values

* Dates: 03/16/05 - Written by D. Boore
*        07/24/05 - Added index j for end cases

      real x(*), y(*), slope, x_intrp, y_intrp
      integer j, n
      
      if (x_intrp .le. x(1)) then
        j = 1
        y_intrp = y(1)
        return
      end if
      
      if (x_intrp .ge. x(n)) then
        j = n
        y_intrp = y(n)
        return
      end if          
      
      call locate(x,n,x_intrp,j)
      
      slope = (y(j+1) - y(j))/(x(j+1)-x(j))
      y_intrp = y(j) + slope*(x_intrp - x(j))
      
      return
      end
      


!* --------------------- BEGIN LOCATE -----------------
      SUBROUTINE locate(xx,n,x,j)
      
* Comments added by D. Boore on 26feb2010:
*  finds j such that xx(j) < x <= xx(j+1)
*  EXCEPT if x = xx(1), then j = 1 (logically it would be 0 from
*  the above relation, but this is the same returned value of j
*  for a point out of range).
*  Also, if x = xx(n), j = n-1, which is OK
*  Note that j = 0 or j = n indicates that x is out of range.
*
* See the program test_locate.for to test this routine.

      INTEGER j,n
      REAL x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).ge.xx(1)).eqv.(x.ge.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      if(x.eq.xx(1))then
        j=1
      else if(x.eq.xx(n))then
        j=n-1
      else
        j=jl
      endif
      return
      END
!* --------------------- END LOCATE -----------------

! ------------------------------------------------  site_amp_correction
      subroutine site_amp_correction(
     :      per, v30, v30ref, 
     :      t4pgv,
     :      imodel_lin, 
     :      bv, va,
     :      nonlin_corr, imodel_nl, pga_4nl,
     :      a1, pga_g_low, a2,
     :      bnl, amp_lin, amp_nonlin, amp_total)
      
! Input parameters: 
!    per: period for site amp calculation 
!    v30: Vs30 for site amp calculation 
!    v30ref: Vs30 reference value (760 m/s used for BA08)
!    imodel_lin: specify which linear model to use (1=BJF, 2=CS05; 2 used for BA08)
!    nonlin_corr = 'Y' for nonlinear site amp calculation
!    imodel_nl: specify which nonlinear model to use (1=CS; 2=GMA; 3=DMB1; 4=DMB2; 3 used for BA08)
!    a1, pga_g_low, a2: parameters for transition from nonlinear to linear
!                       amplification for small values of pga_g: 
!                       a1 = 0.03, pga_g_low = 0.06, a2 = 0.09g used for BA08 NGA GMPEs.

! Output parameters: bv, va, amp_lin
! Output parameters: bnl, amp_nonlin, amp_total (=amp_lin*amp_nonlin)
!                    The amps are for Y, not log Y.

! Use site_amp_from_v30_vs_per.for to compute the site amp for a
! specified model, pga4nl, and Vs30 vs period.

! Dates: 08/02/05 - Written by D. Boore
!        07/03/06 - Add parameters a1, a2 for cubic smoothing
!        08/05/09 - Edited some of the comments to make clearer the
!                   input parameters used in the BA NGA GMPEs.
!        02/26/10 - Improved subroutines to interpolate coefficients.
!        01/05/12 - Include called subprograms rather than appending them here
!                   (to avoid multiple versions, in case changes are made).
!        01/06/12 - Change "pga_g" to "pga_4nl"
!                 - Do not compute nonlinear amps if pga_4nl <= 0.0, even if
!                   nonlin_corr == 'Y'   
!        11/10/12 - Call SS12 model if imodel_lin = 5 or imodel_nl = 5
!        11/26/12 - Add t4pgv as an argument
!        03/16/13 - Change SS12 to SS13
!        03/17/13 - No longer need t4pgv for BSS13
      implicit none
      
      real :: per, v30, v30ref, bv, va, pga_4nl,
     :        a1, pga_g_low, a2,
     :        bnl, amp_lin, amp_nonlin, amp_total,
     :        t4pgv, phi2, phi3
     
      integer :: imodel_lin, imodel_nl
      
      character nonlin_corr*1
 
      if (imodel_lin == 5) then      
        call site_amp_linear_ss13(per, v30, v30ref,
     :                            bv, va, amp_lin)      
      else
        call site_amp_linear(per, v30, v30ref, 
     :                       t4pgv,
     :                       imodel_lin, 
     :                       bv, va, amp_lin)     
      end if
     
      if (nonlin_corr == 'Y' .and. pga_4nl > 0.0) then
        if (imodel_nl == 5) then      
          call site_amp_nonlinear_ss13(
     :      per, phi2, phi3, 
     :      v30, pga_4nl,  
     :      bnl, amp_nonlin)     
        else
          call site_amp_nonlinear(
     :    per, v30, v30ref, 
     :    t4pgv,
     :    pga_4nl, imodel_nl,
     :    a1, pga_g_low, a2,
     :    bnl, amp_nonlin)     
        end if
      else
        bnl = 0.0
        amp_nonlin = 1.0
      end if
      
      amp_total = amp_lin * amp_nonlin
      
      return
      
      end
! ------------------------------------------------  site_amp_correction

      include '\forprogs\site_amp_linear.for'
      include '\forprogs\site_amp_nonlinear.for'
      include '\forprogs\site_amp_linear_ss13.for'
      include '\forprogs\site_amp_nonlinear_ss13.for'
      
!      include '\forprogs\nonlin_b.for'
      
  
      
 
  
      
       